/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
/* vector.c: A set of routines for handling
   2d vectors */

#include <stdio.h>
#include <math.h>

#include "vector.h"

/* return a vector structure from x and y args */

vector2d v2d(REAL x, REAL y) {
  vector2d v;
  v.x = x;
  v.y = y;
  return v;
}

/* is x close to zero? */
REAL reallyzero(REAL x) {
  return (fabs(x) < NOTHING);
}

/* norm of a vector */
REAL norm2d(vector2d v) {
  return sqrt(v.x * v.x + v.y * v.y);
}


/* dot product */
REAL dot2d(vector2d v, vector2d w) {
  return v.x * w.x + v.y * w.y;
}

/* vector sum */
vector2d sum2d(vector2d v, vector2d w) {
  vector2d r;
  r.x = v.x + w.x; r.y = v.y + w.y;
  return r;
}

/* vector product by scalar */
vector2d mul2d(REAL a, vector2d v) {
  vector2d r;
  r.x = v.x * a;
  r.y = v.y * a;
  return r;
}

/* subtract w from v */
vector2d sub2d(vector2d v, vector2d w) {
  vector2d r;
  r.x = v.x - w.x;
  r.y = v.y - w.y;
  return r;
}

/* distance between two vectors */
REAL dist2d(vector2d v, vector2d w) {
  return norm2d(sub2d(v,w));
}

/* 2d determinant of [v,w] */
REAL det2d(vector2d v, vector2d w) {
  return (v.x * w.y - v.y * w.x);
}

/* is v = (0,0)? */
BOOL zero2d(vector2d v) {
  return (v.x==0.0 && v.y == 0.0);
}

/* is v almost zero */
BOOL reallyzero2d(vector2d v) {
  return dot2d(v,v) < (NOTHING*NOTHING);
}

/* are v and w parallel */
BOOL parallel2d(vector2d v, vector2d w) {
  return (det2d(v,w) == 0);
}


/* calculate the torque exerted at mass by a force 
   f applied at point x */
torque2d torqueat(vector2d mass, vector2d x, force2d f) {
  REAL l,tau;
  vector2d p;
  if (parallel2d(f,x)) return 0.0 ;
  l = (-dot2d(x,f) + dot2d(mass,f)) / dot2d(f,f);
  p = sum2d(x,mul2d(l,f));
  tau = norm2d(sub2d(mass,p))*norm2d(f);
  if (det2d(sub2d(p,mass),f) <= 0.0) tau = -tau;
#ifdef PRINTS
  printf("torqueat %f %f x= %f %f f=%f %f returns %f\n",mass.x,mass.y,x.x,x.y,f.x,f.y,tau);
#endif
  return tau;
}



